package microbasic.basic;

import java.util.*;

import microbasic.forms.*;
import microbasic.misc.*;

public class VirtualMachine extends Thread {
    public void init(String program_code) {
        String code = program_code.trim();
        code += "\n";
        code_to_object(code);
    }

    private void code_to_object(String codetext) {
        int lines_count = 0;
        for (int last = codetext.indexOf('\n'); last != -1;
                        last = codetext.indexOf('\n', last + 1)) {
            lines_count++;
        }
        code_args = new String[lines_count];
        code_commands = new int[lines_count];
        for (int x = 0; x != code_commands.length; x++) {
            code_commands[x] = -1;
        }
        Commands.clear_marks();
        /* Zaczynam transformowac */
        StringIterator lines = new StringIterator("\n", codetext);
        int current_line = 0;
        try {
            while (lines.hasMoreTokens()) {
                /* Wczytuje kolejne linie */
                String line = lines.nextToken();
                line = Util.btrim(line).trim();
                if (line.length() == 0) {
                    continue;
                }
                StringIterator args = new StringIterator(" ", line);
                /* Komenda */
                String command = args.nextToken();
                int num = Commands.command_code(command);
                if (num == -1) {
                    throw new Exception("No such command at line " +
                                        current_line);
                } else if (num == 7) { //Mark
                    if (line.length() > (command.length() + 1)) {
                        String arg = Util.btrim(line.substring(command.length() +
                                1,
                                line.length()).trim());
                        Commands.addMark(arg, current_line);
                        current_line--;
                    } else {
                        throw new Exception(
                                "Mark command needs argument at line " +
                                current_line);
                    }
                } else {
                    code_commands[current_line] = num;
                    if (line.length() > (command.length() + 1)) {
                        String arg = Util.btrim(line.substring(command.length() +
                                1,
                                line.length()));
                        if (arg.length() != 0) {
                            code_args[current_line] = arg;
                        } else {
                            code_args[current_line] = null;
                        }
                    }
                }
                current_line++;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* Najwazniejsza funkcja ! */
    public void run() {
        Commands.set_vm(this);
        Memory.clear();
        try {
            Memory.alloc("gwdt");
            Memory.setVariable("gwdt", new Integer(RunForm.instance.width));
            Memory.alloc("ghgt");
            Memory.setVariable("ghgt", new Integer(RunForm.instance.height));
            Memory.alloc("red");
            Memory.setVariable("red", new Integer(1<<16));
            Memory.alloc("green");
            Memory.setVariable("green", new Integer(1<<8));
            Memory.alloc("blue");
            Memory.setVariable("blue", new Integer(1));
        } catch (Exception ex) {
        }
        try {
            for (line = 0;
                        line != code_commands.length &&
                        code_commands[line] != -1;
                        line++) {
                if (stop) {
                    return;
                }
                Commands.run_command(code_commands[line], code_args[line], line);
                Thread.sleep(10);
            }
        } catch (InterruptedException in) {

        } catch (Exception e) {
            RunForm.println(String.valueOf(line + 1) + ": " + e.getMessage());
        }

    }

    public void interrupt() {
        stop = true;
    }

    public void set_line(int _line) {
        line = _line;
    }

    int line;
    int[] code_commands = null;
    String[] code_args = null;
    public boolean stop = false;
}


class Commands {
    static public void set_vm(VirtualMachine _vm) {
        vm = _vm;
    }

    static public void run_command(int comm, String arg, int line) throws
            Exception {
        switch (comm) {
        case 0:
            var(arg);
            break;
        case 1:
            let(arg);
            break;
        case 2:
            print(arg);
            break;
        case 3:
            input(arg);
            break;
        case 4:
            RunForm.cls();
            break;
        case 5:
            if_c(arg, line);
            break;
        case 6:
            else_c(arg, line);
            break;
        case 7:
            break;
        case 8:
            goto_c(arg);
            break;
        case 9:
            for_c(arg, line);
            break;
        case 10:
            next(arg);
            break;
        case 11:
            gosub(arg, line);
            break;
        case 12:
            return_c(arg);
            break;
        case 13:
            gclr();
            break;
        case 14:
            RunForm.draw(0, arg);
            break;
        case 15:
            RunForm.draw(1, arg);
            break;
        case 16:
            RunForm.draw(2, arg);
            break;
        case 17:
            RunForm.draw(3, arg);
            break;
        case 18:
            RunForm.draw(4, arg);
            break;
        default:
            throw new Exception("No such command!");
        }
    }

    /* Funkcje */
    static private void var(String arg) throws Exception {
        StringIterator iter = new StringIterator(",", arg);
        while (iter.hasMoreTokens()) {
            String statement = iter.nextToken();
            String var_s;
            boolean eq;
            if (statement.indexOf('=') != -1) {
                var_s = statement.substring(0, statement.indexOf('='));
                eq = true;
            } else {
                var_s = statement;
                eq = false;
            }
            Memory.alloc(var_s.trim());
            if (eq) {
                let(statement);
            }
        }
    }

    static private void let(String _arg) throws Exception {
        StringIterator main = new StringIterator(",", _arg);
        while (main.hasMoreTokens()) {
            String arg = Util.btrim(main.nextToken()).trim();
            StringIterator iter = new StringIterator("=", arg);
            String left = iter.nextToken();
            String right = arg.substring(left.length() + 1,
                                         arg.length());
            if (!left.endsWith("$")) { // Number
                Equation eq = new Equation();
                int right_res = eq.solve(right);
                Memory.setVariable(left, new Integer(right_res));
            } else { // String
                Memory.setVariable(left, Equation.solve_string_equation(right));
            }
        }
    }

    static private void print(String arg) throws Exception {
        Vector objs = new Vector();
        StringIterator semicolon_iter = new StringIterator(";", arg);
        while (semicolon_iter.hasMoreTokens()) {
            objs.addElement(semicolon_iter.nextToken());
        }

        Enumeration en = objs.elements();
        while (en.hasMoreElements()) {
            String current = (String) en.nextElement();
            if (current.charAt(0) == '\"' &&
                current.charAt(current.length() - 1) == '\"') { // String
                RunForm.print(current.substring(1, current.length() - 1));
            } else if (current.indexOf('$') != -1) { // a$
                RunForm.print(Equation.solve_string_equation(current));
            } else { // number
                Equation eq = new Equation();
                RunForm.print(String.valueOf(eq.solve(current)));
            }
        }
        RunForm.println("");
    }

    static private void input(String arg) throws Exception {
        arg = arg.trim();
        if (arg.indexOf(';') == -1) {
            throw new Exception("No destination variable!");
        }
        String var = arg.substring(arg.lastIndexOf(';') + 1);
        String req = arg.substring(0, arg.lastIndexOf(';'));
        print(req);
        if (var.charAt(var.length() - 1) == '$') { //String
            String ret = RunForm.read(null, false);
            Memory.setVariable(var, ret);
            print("\"? " + ret + "\"");
        } else { //Number
            String ret = RunForm.read(null, true);
            Memory.setVariable(var, Integer.valueOf(ret));
            print("\"? " + ret + "\"");
        }
    }

    static private void if_c(String arg, int num) throws Exception {
        // Dzielimy if cos[znak]cos then cos
        StringIterator space_iter = new StringIterator(" ", arg);
        String then = "", eq = null;
        try {
            /*            eq = space_iter.nextToken();
                        if (!space_iter.nextToken().equals("then")) {
                            throw new Exception();
                        } while (space_iter.hasMoreTokens()) {
                            then += space_iter.nextToken() + " ";
                        }
                        then = then.trim();*/
            if (arg.indexOf("then") == -1) {
                throw new Exception();
            }

            eq = Util.btrim(arg.substring(0, arg.indexOf("then") - 1).trim());
            then = Util.btrim(arg.substring(arg.indexOf("then") + 5).trim());

            if (if_check(eq)) {
                start_command(then, num);
            }
        } catch (Exception e) {
            throw new Exception("Parse error!");
        }
    }

    static private void start_command(String line, int num) throws Exception {
        String command = "";
        String arg = "";
        if (line.indexOf(' ') == -1 || line.indexOf(' ') + 1 == line.length()) {
            command = line;
        } else {
            command = line.substring(0, line.indexOf(' '));
            arg = line.substring(line.indexOf(' ') + 1);
        }
        run_command(command_code(command), arg, num);
    }

    static private boolean if_check(String eq) throws Exception {
        String[] ops = {"!=", ">=", "<=", "=", ">", "<"};
        String op = "!=";
        String left, right;
        int index = -1;
        for (int x = 0; x != 6; x++) {
            if ((index = eq.indexOf(ops[x])) != -1) {
                op = ops[x];
                break;
            }
        }
        left = eq.substring(0, index);
        right = eq.substring(index + op.length());
        if (left.indexOf('\"') != -1 || left.indexOf('$') != -1) { //String
            left = Equation.solve_string_equation(left);
            right = Equation.solve_string_equation(right);
            return (last_if = if_c_compare_string(left, right, op));

        } else { //Number
            Equation eq1 = new Equation();
            Equation eq2 = new Equation();
            int n1 = eq1.solve(left);
            int n2 = eq2.solve(right);
            return (last_if = if_c_compare_int(n1, n2, op));
        }
    }

    static private boolean if_c_compare_string(String left, String right,
                                               String op) {
        if (op.equals("=")) {
            if (left.equals(right)) {
                return true;
            }
        } else if (op.equals("<")) {
            if (left.length() < right.length()) {
                return true;
            }
        } else if (op.equals(">")) {
            if (left.length() > right.length()) {
                return true;
            }
        } else if (op.equals("!=")) {
            if (!left.equals(right)) {
                return true;
            }
        } else if (op.equals(">=")) {
            if (left.length() >= right.length()) {
                return true;
            }
        } else if (op.equals("<=")) {
            if (left.length() <= right.length()) {
                return true;
            }
        }

        return false;
    }

    static private boolean if_c_compare_int(int left, int right,
                                            String op) {
        if (op.equals("=")) {
            if (left == right) {
                return true;
            }
        } else if (op.equals("<")) {
            if (left < right) {
                return true;
            }
        } else if (op.equals(">")) {
            if (left > right) {
                return true;
            }
        } else if (op.equals("!=")) {
            if (left != right) {
                return true;
            }
        } else if (op.equals(">=")) {
            if (left >= right) {
                return true;
            }
        } else if (op.equals("<=")) {
            if (left <= right) {
                return true;
            }
        }
        return false;
    }

    static private void else_c(String arg, int num) throws Exception {
        if (!last_if) {
            start_command(arg, num);
        }
    }

    static private void goto_c(String arg) throws Exception {
        int line = ((Integer) marks.get(arg.trim())).intValue();
        vm.set_line(line - 1);
    }

    static private void for_c(String arg, int line) throws Exception {
        StringIterator iter = new StringIterator(" ", arg);
        String fors, to = "";
        int to_int;
        /*
                fors = iter.nextToken();
                if (!iter.nextToken().equals("to")) {
                    throw new Exception("No \'to\' statement!");
                } while (iter.hasMoreTokens()) {
                    to += iter.nextToken() + " ";
                }
         */
        if (arg.indexOf("to") == -1) {
            throw new Exception();
        }

        fors = Util.btrim(arg.substring(0, arg.indexOf("to") - 1).trim());
        to = Util.btrim(arg.substring(arg.indexOf("to") + 3).trim());
        Equation to_eq = new Equation();
        to_int = to_eq.solve(to.trim());
        StringIterator eqiter = new StringIterator("=", fors);
        String left = eqiter.nextToken();
        String right = fors.substring(left.length() + 1,
                                      fors.length());
        Equation eq = new Equation();
        int right_res = eq.solve(right);
        Memory.setVariable(left, new Integer(right_res));

        formarks.put(left, new Integer[] {new Integer(to_int), new Integer(line)});
    }

    static private void next(String arg) throws Exception {
        Integer[] data = (Integer[]) formarks.get(arg);
        Integer var = (Integer) Memory.getVariable(arg);
        var = new Integer(var.intValue() + 1);
        Memory.setVariable(arg, var);
        if (var.intValue() != data[0].intValue()) {
            vm.set_line(data[1].intValue());
        }
    }

    static private void gosub(String arg, int line) throws Exception {
        gosubmarks.addElement(new Integer(line));
        goto_c(arg);
    }

    static private void return_c(String arg) throws Exception {
        if (gosubmarks.isEmpty()) {
            throw new Exception("Use gosub first!");
        }
        int line = ((Integer) gosubmarks.lastElement()).intValue();
        gosubmarks.removeElementAt(gosubmarks.size() - 1);
        vm.set_line(line);
    }

    static private void gclr() throws Exception {
        RunForm.gclr();
    }


    static public void addMark(String name, int line) {
        marks.put(name, new Integer(line));
    }

    static public void clear_marks() {
        marks.clear();
        formarks.clear();
        gosubmarks.removeAllElements();
    }

    static public int command_code(String command) {
        for (int x = 0; x != Commands.commands.length; x++) {
            if (Commands.commands[x].equals(command)) {
                return x;
            }
        }
        return -1;
    }

    static Hashtable marks = new Hashtable();
    static Hashtable formarks = new Hashtable(3);
    static Vector gosubmarks = new Vector(5);
    static boolean last_if;
    static private VirtualMachine vm;
    static String[] commands = {"var", "let", "print", "input", "clr", "if",
                               "else", "mark", "goto", "for", "next", "gosub",
                               "return", "gclr", "gprint", "rect", "circle",
                               "line", "poly"};
}
