package microbasic.forms;

import javax.microedition.lcdui.*;

import java.util.Vector;
import java.util.Enumeration;
import microbasic.basic.VirtualMachine;
import microbasic.MicroBasic;
import microbasic.forms.MainForm;
import microbasic.misc.*;
import microbasic.basic.Equation;


public class RunForm {
    public RunForm(String code) {
        instance = this;

        vm = new VirtualMachine();
        vm.init(code);

        text = new TextRunForm(vm);
        graph = new GraphicRunForm(vm);
        height = graph.getHeight();
        width = graph.getWidth();
        vm.start();

    }

    static public void cls() {
        instance.text.cls();
    }

    static public Integer getKey() {
        Integer ret = null;
        while ((ret = instance.graph.getKey()) == null && !instance.vm.stop) {
            ;
        }
        return ret;
    }


    static public void print(String txt) {
        instance.text.print(txt);

    }

    static public void println(String txt) {
        instance.text.print(txt+"\n");
    }

    static public String read(String req, boolean number) {
        return instance.text.read(req, number);
    }

    static public void draw(int type, String arg) throws Exception {
        GraphObject object = null;
        Vector args = new Vector(8);
        Exception ex = new Exception("Bad args number!");
        StringIterator iter = new StringIterator(",", arg);
        while (iter.hasMoreTokens()) {
            args.addElement(Util.btrim(iter.nextToken()).trim());
        }

        if (type == 0 && args.size() < 3) {
            throw ex;
        } else if (type >= 1 && type <= 2 && args.size() < 5) {
            throw ex;
        } else if (type == 3 && args.size() < 4) {
            throw ex;
        } else if (type == 4 && args.size() < 7) {
            throw ex;
        }

        Enumeration en = args.elements();

        switch (type) {
        case GraphObject.STRING:
            object = new GraphObject(type,
                                     new int[] {(new Equation()).solve((
                                             String) en.nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement())}, Equation.solve_string_equation((String)
                    en.nextElement()));
            if (en.hasMoreElements()) {
                object.setColor((new Equation()).solve((String) en.nextElement()));
            }
            break;
        case GraphObject.RECTANGLE:
            object = new GraphObject(type,
                                     new int[] {(new Equation()).solve((
                                             String) en.nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement())},
                                     (new Equation()).solve((String) en.
                    nextElement()));
            if (en.hasMoreElements()) {
                object.setColor((new Equation()).solve((String) en.nextElement()));
            }
            break;
        case GraphObject.CIRCLE:
            object = new GraphObject(type,
                                     new int[] {(new Equation()).solve((
                                             String) en.nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement())},
                                     (new Equation()).solve((String) en.
                    nextElement()));
            if (en.hasMoreElements()) {
                object.setColor((new Equation()).solve((String) en.nextElement()));
            }
            break;
        case GraphObject.LINE:
            object = new GraphObject(type,
                                     new int[] {(new Equation()).solve((
                                             String) en.nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement())});
            if (en.hasMoreElements()) {
                object.setColor((new Equation()).solve((String) en.nextElement()));
            }
            break;
        case GraphObject.POLY:
            object = new GraphObject(type,
                                     new int[] {(new Equation()).solve((
                                             String) en.nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement()),
                                     (new Equation()).solve((String) en.
                    nextElement())
            },
                    (new Equation()).solve((String) en.
                                           nextElement()));
            if (en.hasMoreElements()) {
                object.setColor((new Equation()).solve((String) en.nextElement()));
            }
            break;
        }
        instance.graph.draw(object);
        instance.graph.repaint();
    }

    static public void gclr() {
        instance.graph.gclr();
        instance.graph.repaint();

    }

    private VirtualMachine vm;
    public TextRunForm text;
    public GraphicRunForm graph;
    public int height, width;
    static public RunForm instance;
}


class TextRunForm extends Form implements CommandListener {
    public TextRunForm(VirtualMachine _vm) {
        super("Running program...");

        addCommand(new Command("Exit", Command.EXIT, 0));
        setCommandListener(this);

        text = new StringItem(null, "");
        append(text);

        read_value = null;
        vm = _vm;
    }


    public void commandAction(Command command, Displayable displayable) {
        if (command.getCommandType() == command.EXIT) {
            vm.interrupt();
            Display.getDisplay(MicroBasic.instance).setCurrent(CodeForm.
                    instance);
        } else if (command.getCommandType() == command.OK) {
            read_value = text_field.getString();
            removeCommand(enter);
        }
    }

    public void cls() {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        text.setText("");
    }

    public void print(String txt) {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        text.setText(text.getText() + txt);
    }

    public void println(String txt) {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        text.setText(text.getText() + txt + "\n");
    }

    public String read(String req, boolean number) {
        init_read(req, number);
        while (read_value == null) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException ex) {
            }
        }
        delete(size() - 1);
        return read_value;
    }

    private void init_read(String req, boolean number) {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        read_value = null;
        enter = new Command("Enter", Command.OK, 0);
        addCommand(enter);
        if (number) {
            text_field = new TextField(req, null, 256, TextField.NUMERIC);
        } else {
            text_field = new TextField(req, null, 256, TextField.ANY);
        }
        append(text_field);
    }

    private StringItem text;
    private VirtualMachine vm;
    static private String read_value;
    static private TextField text_field;
    static private Command enter;
}


class GraphicRunForm extends Canvas implements CommandListener {
    public GraphicRunForm(VirtualMachine _vm) {
        addCommand(new Command("Exit", Command.EXIT, 0));
        setCommandListener(this);

        gbuf = null;
        keybuf = new Vector();
        vm = _vm;
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command.getCommandType() == command.EXIT) {
            vm.interrupt();
            Display.getDisplay(MicroBasic.instance).setCurrent(CodeForm.
                    instance);
        }
    }

    public void paint(Graphics g) {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        if (gbuf != null) {
            gbuf.draw(g);
        } else {
            g.setColor(255, 255, 255);
            g.fillRect(0, 0, this.getWidth(), this.getHeight());
        }
    }

    public void gclr() {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        gbuf = null;
        repaint();
    }

    public Integer getKey() {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        if (keybuf.isEmpty()) {
            return null;
        } else {
            Integer i = (Integer) keybuf.elementAt(0);
            keybuf.removeElementAt(0);
            return i;
        }
    }

    public void draw(GraphObject obj) {
        Display.getDisplay(MicroBasic.instance).setCurrent(this);
        gbuf = obj;
    }

    protected void keyPressed(int key) {
        keybuf.addElement(new Integer(key));
    }

    private Vector keybuf;
    private GraphObject gbuf;
    private VirtualMachine vm;
}


class GraphObject {

    public GraphObject(int _type, int _coor[]) {
        type = _type;
        coor = _coor;
        fill = 0;
        color = -1;
        txt = null;
    }

    public GraphObject(int _type, int _coor[], String _txt) {
        type = _type;
        coor = _coor;
        fill = 0;
        txt = _txt;

    }


    public GraphObject(int _type, int _coor[], int _fill) {
        type = _type;
        coor = _coor;
        fill = _fill;
        color = -1;
    }

    public void draw(Graphics g) {
        if (color != -1) {
            g.setColor(color);
        }
        switch (type) {
        case STRING:
            g.drawString(txt, coor[0], coor[1], g.BASELINE | g.LEFT);
            break;
        case RECTANGLE:
            if (fill != 0) {
                g.fillRect(coor[0], coor[1], coor[2], coor[3]);
            } else {
                g.drawRect(coor[0], coor[1], coor[2], coor[3]);
            }
            break;
        case CIRCLE:
            if (fill != 0) {
                g.fillArc(coor[0], coor[1], coor[2], coor[3], 0, 360);
            } else {
                g.drawArc(coor[0], coor[1], coor[2], coor[3], 0, 360);
            }
            break;
        case LINE:
            g.drawLine(coor[0], coor[1], coor[2], coor[3]);
            break;
        case POLY:
            if (fill != 0) {
                g.fillTriangle(coor[0], coor[1], coor[2], coor[3], coor[4],
                               coor[5]);
            } else {
                g.drawLine(coor[0], coor[1], coor[2], coor[3]);
                g.drawLine(coor[2], coor[3], coor[4], coor[5]);
                g.drawLine(coor[0], coor[1], coor[4], coor[5]);
            }
            break;
        }
    }

    public void setColor(int _color) {
        color = _color;
    }

    private String txt;
    private int fill;
    private int coor[];
    private int type;
    private int color = -1;
    static final public int STRING = 0, RECTANGLE = 1, CIRCLE = 2, LINE = 3,
            POLY = 4;
}
