package microbasic.forms;

import java.io.*;

import javax.microedition.lcdui.*;

import microbasic.*;
import microbasic.misc.*;

public class SaveForm extends Form implements CommandListener {
    public SaveForm(String _code) {
        super("Save file...");

        fs = new Stock("fs");
        fs.open();
        filename_visible = false;
        code = _code;

        stock_size = fs.length();

        addCommand(new Command("Save", Command.SCREEN, 0));
        addCommand(new Command("Format", Command.SCREEN, 1));
        addCommand(new Command("Delete", Command.SCREEN, 2));
        addCommand(new Command("Exit", Command.EXIT, 3));
        setCommandListener(this);
        fillFilenames();
        append(files_item);
    }

    public synchronized void fillFilenames() {
        Image imgnew = null, imgfile = null;
        try {
            imgnew = Image.createImage("/new.png");
            imgfile = Image.createImage("/file.png");
        } catch (IOException ex) {
        }

        files_item = new ChoiceGroup("Files:", Choice.EXCLUSIVE);
        files_item.insert(files_item.size(), "New", imgnew);
        if (stock_size == 0) {
            return;
        }
        for (int x = 0; x < stock_size; x++) {
            String item = fs.get(x);
            try {
                item = item.substring(0, item.indexOf('|'));
                files_item.insert(files_item.size(), item, imgfile);
            } catch (Exception e) {
                System.err.println("Found invalid file! Deleting...");
                fs.delete(x);
                break;
            }
        }
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command.getCommandType() == Command.SCREEN) {
            if (command.getPriority() == 0 && !filename_visible) { // Save
                filename_visible = true;
                int index = files_item.getSelectedIndex();
                if (index == 0) {
                    this.delete(0);
                    filename = new TextField("Enter filename", "", 20,
                                             TextField.ANY);
                    append(filename);
                } else { // Saves As
                    String name = fs.get(index - 1);
                    name = name.substring(0, name.indexOf('|'));
                    fs.set(index - 1, name + "|" + code);
                    fs.close();
                    Display.getDisplay(MicroBasic.instance).setCurrent(MainForm.
                            previous);
                }
            } else if (command.getPriority() == 0 && filename_visible) { //Saves
                if (filename.getString().length() == 0 ||
                    filename.getString().indexOf('|') != -1) {
                    return;
                }
                fs.add(filename.getString() + "|" + code);
                fs.close();
                Display.getDisplay(MicroBasic.instance).setCurrent(MainForm.
                        previous);

            } else if (command.getPriority() == 1 && !filename_visible) { // Format
                fs.delete();
                stock_size = 0;
                fs.close();
                Display.getDisplay(MicroBasic.instance).setCurrent(MainForm.
                        previous);

            } else if (command.getPriority() == 2 && !filename_visible) {
                if (stock_size == 0) {
                    return;
                }
                stock_size--;
                fs.delete(files_item.getSelectedIndex() - 1);
                fs.close();
                Display.getDisplay(MicroBasic.instance).setCurrent(MainForm.
                        previous);

            }
        } else if (command.getCommandType() == Command.EXIT) {
            fs.close();
            Display.getDisplay(MicroBasic.instance).setCurrent(MainForm.
                    previous);
        }
    }

    private int stock_size;
    private boolean filename_visible;
    private Stock fs;
    private String code;
    private TextField filename;
    private ChoiceGroup files_item;
}
