package microbasic.misc;

import javax.microedition.rms.*;
import java.util.Enumeration;
import java.util.Vector;

public class Stock {
    public Stock(String _name) {
        name = _name;
    }

    public synchronized void open() {
        try {
            rs = RecordStore.openRecordStore(name, true);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }

    public synchronized int add(String data) {
        byte bytes[] = data.getBytes();
        try {
            return rs.addRecord(bytes, 0, bytes.length);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
            return -1;
        }
    }

    public synchronized int length() {
        try {
            return rs.getNumRecords();
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        return 0;
    }

    public synchronized void set(int i, String data) {
        byte bytes[] = data.getBytes();
        try {
            int id=0;
            RecordEnumeration en = this.enumerate();
            for(int x=0;x<=i;x++) id=en.nextRecordId();
            rs.setRecord(id, bytes, 0, bytes.length);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }


    public synchronized String get(int i) {
        try {
            int id=0;
            RecordEnumeration en = this.enumerate();
            for(int x=0;x<=i;x++) id=en.nextRecordId();
            byte b[] = rs.getRecord(id);
            String str = new String(b, 0, b.length);
            return str;
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
            return null;
        }
    }

    public synchronized void delete() {
        try {
            this.close();
            rs.deleteRecordStore(name);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }

    public synchronized void delete(int i) {
        try {
            int id=0;
            RecordEnumeration en = this.enumerate();
            for(int x=0;x<=i;x++) id=en.nextRecordId();
            rs.deleteRecord(id);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }

    public synchronized RecordEnumeration enumerate() {
        try {
            return rs.enumerateRecords(null, null, false);
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
            return null;
        }
    }

    public synchronized void close() {
        try {
            rs.closeRecordStore();
        } catch (RecordStoreException e) {
            System.err.println(e);
            e.printStackTrace();
        }
    }

    public RecordStore rs;
    private String name;
}
